@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Config.pm,v 1.81.2.72 2005/06/02 17:08:08 jkf Exp $
+#   $Id: Config.pm,v 1.81.2.77 2006/01/15 19:52:10 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.81.2.72 $, 10;
+$VERSION = substr q$Revision: 1.81.2.77 $, 10;
 
 #1 exception to general rule of forming replies to Config::Value()
 #is %Config::ScannerCmds
@@ -559,8 +559,13 @@ sub initialise {
   }
 
   while(defined($filename = $dirh->read)) {
+    # Only process files ending with .pm or .pl
     # Skip all dot files and rpmnew files
     next if $filename =~ /^\./ || $filename =~ /\.(rpmnew|dpkg-dist|dpkg-new|dpkg-old)$/i;
+    unless ($filename =~ /\.p[lm]$/i) {
+      MailScanner::Log::InfoLog("Skipping Custom Function file %s as its name does not end in .pm or .pl", $filename);
+      next;
+    }
     $fullfile = "$dir/$filename";
     next unless -f $fullfile and -s $fullfile;
     eval { require $fullfile; };
@@ -580,13 +585,13 @@ sub initialise {
 # Cannot use MailScanner::Log here at all, as it hasn't started yet.
 #
 sub QuickPeek {
-  my($filename, $target) = @_;
+  my($filename, $target, $notifldap) = @_;
 
   my($fh, $key, $value, $targetfound, $targetvalue);
   my($ldapserver, $ldapsite, $ldapbase);
 
   $target = lc($target);
-  $target =~ s/[^a-z0-9]//g;
+  $target =~ s/[^%a-z0-9]//g; # Leave % vars intact
 
   $fh = new FileHandle;
   $fh->open("<$filename") or die "Cannot open config file $filename, $!";
@@ -610,11 +615,11 @@ sub QuickPeek {
     $value =~ s/\\n/\n/g;
     if ($key =~ /^\%([^%]+)\%$/) {
       $PercentVars{lc($1)} = $value;
-      next;
+      #next; -- Store the percentvars in the key{value} hash as well.
     }
 
     $key = lc($key);
-    $key =~ s/[^a-z0-9]//g; # Leave numbers and letters only
+    $key =~ s/[^%a-z0-9]//g; # Leave numbers and letters only -- leave % vars
     $ldapserver = $value if $key =~ /ldapserver/i;
     $ldapsite   = $value if $key =~ /ldapsite/i;
     $ldapbase   = $value if $key =~ /ldapbase/i;
@@ -631,6 +636,11 @@ sub QuickPeek {
   flock($fh, $LOCK_UN);
   $fh->close();
 
+  # Now do the stuff if $noldap is set, so we don't actually
+  if ($ldapserver && $notifldap) {
+    return undef;
+  }
+
   #
   # Now try and over-ride the value with the setting from LDAP
   #
@@ -813,7 +823,7 @@ sub Read {
 sub ReadPhishingWhitelist {
   my($filename) = @_;
 
-  my($fh, %whitelist, $counter);
+  my($fh, %whitelist, @blacklist, $counter);
 
   %whitelist = ();
 
@@ -839,8 +849,20 @@ sub ReadPhishingWhitelist {
     s/\s*$//g;   # Remove trailing white space
     s/\s+.*$//g; # Leave only the 1st word
     next if /^$/;
-    $whitelist{$_} = 1;
-    $counter++;
+    # Entries in the list starting with "REMOVE " in capitals cause the entry
+    # to be forcibly removed from the phishing whitelist.
+    if (/^REMOVE\s+(\S+)/i) {
+      delete $whitelist{$1};
+      push @blacklist, $1;
+    } else {
+      $whitelist{$_} = 1;
+      $counter++;
+    }
+  }
+
+  # Now process the blacklist
+  foreach (@blacklist) {
+    delete $whitelist{$_};
   }
 
   close $fh;
@@ -880,6 +902,7 @@ sub FilenameRulesValue {
   # Get the list of filenames and split it
   $list = Value('filenamerules', $message);
   @filenamelist = split(" ", $list);
+  return undef unless @filenamelist;
 
   # Now construct a list containing the concatenation of all the allow-deny
   # rules
@@ -915,6 +938,7 @@ sub FiletypeRulesValue {
   # Get the list of filenames and split it
   $list = Value('filetyperules', $message);
   @filetypelist = split(" ", $list);
+  return undef unless @filetypelist;
 
   # Now construct a list containing the concatenation of all the allow-deny
   # rules
@@ -1531,6 +1555,9 @@ sub ReadConfBasicLDAP {
     }
     $number++;
   }
+
+  # SF - Read the rest of the LDAP config
+  MailScanner::Config::LDAPUpdated();
 }
 
 
@@ -2754,6 +2781,7 @@ sub LDAPUpdated {
     return 0;
   }
 
+  # The first time around, the expiry time will be 0 so it will trigger.
   my $now = time;
   if ($now>$LDAPSerialExpires) {
     # Serial number has expired, fetch a new one
