@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl,v 1.37.2.93 2005/08/27 16:03:38 jkf Exp $
+#   $Id: ConfigDefs.pl,v 1.37.2.107 2006/02/23 21:04:48 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -63,6 +63,7 @@ bayeswait			= WaitDuringBayesRebuild
 blacklistedishigh		= definitespamishighscoring
 BlockEncrypted			= BlockEncryptedMessages
 BlockUnencrypted		= BlockUnencryptedMessages
+cachetiming			= SpamAssassinCacheTimings
 CheckSAIfOnSpamList		= checkspamassassinifonspamlist
 children			= maxchildren
 clamavmaxreclevel		= clamavmodulemaximumrecursionlevel
@@ -112,10 +113,12 @@ inlinetextsig			= inlinetextsignature
 inlinetextwarning		= inlinetextwarning
 inqueuedir			= incomingqueuedir
 keepspamarchiveclean		= keepspamandmcparchiveclean
+lastafterbatch			= alwayslookeduplastafterbatch
 lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
 loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
+lookforuu			= finduuencodedfiles
 maxattachmentsize		= maximumattachmentsize
 maxdirtybytes			= maxunsafebytesperscan
 maxdirtymessages		= maxunsafemessagesperscan
@@ -145,9 +148,12 @@ quarantineuser			= quarantineuser
 quarantinegroup			= quarantinegroup
 quarantineperms			= quarantinepermissions
 rbltimeoutlen			= spamlisttimeoutshistory
+usesacache			= cachespamassassinresults
 satimeoutlen			= spamassassintimeoutshistory
 removeheaders			= removetheseheaders
+replacetnef			= usetnefcontents
 reqspamassassinscore		= requiredspamassassinscore
+sacache				= spamassassincachedatabasefile
 saviwatchfiles                  = monitorsforsophosupdates
 scanmail			= scanmessages
 scoreformat			= spamscorenumberformat
@@ -234,12 +240,12 @@ lockfiledir		/tmp
 
 # Check the first word of these for file existence
 [Simple,File]
-PhishingWhitelist	/opt/MailScanner/etc/phishing.safe.sites.conf
+PhishingWhitelist	/etc/MailScanner/phishing.safe.sites.conf
 pidfile			/var/run/MailScanner.pid
-spamassassinprefsfile	/opt/MailScanner/etc/spam.assassin.prefs.conf
-SpamListDefinitions	/opt/MailScanner/etc/spam.lists.conf
-mcpspamassassinprefsfile /opt/MailScanner/etc/mcp/mcp.spam.assassin.prefs.conf
-VirusScannerDefinitions	/opt/MailScanner/etc/virus.scanners.conf
+#spamassassinprefsfile	/etc/MailScanner/spam.assassin.prefs.conf
+SpamListDefinitions	/etc/MailScanner/spam.lists.conf
+mcpspamassassinprefsfile /etc/MailScanner/mcp/mcp.spam.assassin.prefs.conf
+VirusScannerDefinitions	/etc/MailScanner/virus.scanners.conf
 
 # Check these to ensure they are just numbers
 [Simple,Number]
@@ -279,8 +285,9 @@ WhitelistMaxRecips		20
 # inqueuedir is here as it can be a glob (if it contains a * or a ?) or a
 # filename containing a list of directories.
 [Simple,Other]
+cachetiming		1800,1800,10800,172800,1200
 ClamWatchFiles		/usr/local/share/clamav/*.cvd
-CustomFunctionsDir	/opt/MailScanner/lib/MailScanner/CustomFunctions
+CustomFunctionsDir	/usr/lib/MailScanner/MailScanner/CustomFunctions
 FileCommand		/usr/bin/file
 FirstCheck		mcp
 inqueuedir		/var/spool/mqueue.in
@@ -298,6 +305,7 @@ QuarantineGroup
 QuarantinePerms		0600
 RunAsUser		0
 RunAsGroup		0
+SACache			/var/spool/MailScanner/incoming/SpamAssassin.cache.db
 SAVIWatchFiles		/usr/local/Sophos/ide/*.zip
 SophosAllowedErrors	
 sophoside		
@@ -309,10 +317,10 @@ SpamAssassinDefaultRulesDir
 SpamAssassinInstallPrefix	
 SpamStarsCharacter	s
 MCPSpamAssassinUserStateDir
-MCPSpamAssassinLocalRulesDir	/opt/MailScanner/etc/mcp
-MCPSpamAssassinDefaultRulesDir	/opt/MailScanner/etc/mcp
-MCPSpamAssassinInstallPrefix	/opt/MailScanner/etc/mcp
-TNEFExpander		/opt/MailScanner/bin/tnef --maxsize=100000000
+MCPSpamAssassinLocalRulesDir	/etc/MailScanner/mcp
+MCPSpamAssassinDefaultRulesDir	/etc/MailScanner/mcp
+MCPSpamAssassinInstallPrefix	/etc/MailScanner/mcp
+TNEFExpander		/usr/bin/tnef --maxsize=100000000
 UnrarCommand		/usr/bin/unrar
 VirusScanners		none  # Space-separated list
 WorkUser
@@ -347,12 +355,14 @@ HighSpamPrependSubject	1	no	0	yes	1
 IncludeSpamHeader	0	no	0	yes	1
 IncludeMCPHeader	0	no	0	yes	1
 KeepSpamArchiveClean	1	no	0	yes	1
+LastAfterBatch		0	no	0	yes	1
 LastLookup		0	no	0	yes	1
 ListSAScores		0	no	0	yes	1
 #LoadSpamAssassin	0	no	0	yes	1
 LogHTMLTags		0	no	0	yes	1
 LogPermittedFilenames	0	no	0	yes	1
 LogPermittedFiletypes	0	no	0	yes	1
+LookForUU		0	no	0	yes	1
 MultipleHeaders		append	append	append	replace	replace	add	add
 NamePrependSubject	1	no	0	yes	1
 NoticeFullHeaders	0	no	0	yes	1
@@ -375,6 +385,7 @@ MCPChecks		0	no	0	yes	1
 MCPDetail		1	no	0	yes	1
 MCPListSAScores		0	no	0	yes	1
 MCPWhitelist		0	no	0	yes	1
+UseSACache		1	no	0	yes	1
 VirusPrependSubject	1	no	0	yes	1
 warningisattachment	1	no	0	yes	1
 WarnSenders		0	no	0	yes	1
@@ -383,29 +394,30 @@ WarnNameSenders		1	no	0	yes	1
 WarnOtherSenders	1	no	0	yes	1
 
 [First,File]
-DeletedContentMessage	/opt/MailScanner/etc/reports/en/deleted.content.message.txt
-DeletedFilenameMessage	/opt/MailScanner/etc/reports/en/deleted.filename.message.txt
-DeletedVirusMessage	/opt/MailScanner/etc/reports/en/deleted.virus.message.txt
-DisinfectedReportText	/opt/MailScanner/etc/reports/en/disinfected.report.txt
-inlinehtmlsig		/opt/MailScanner/etc/reports/en/inline.sig.html
-inlinehtmlwarning	/opt/MailScanner/etc/reports/en/inline.warning.html
-inlinespamwarning	/opt/MailScanner/etc/reports/en/inline.spam.warning.txt
-inlinetextsig		/opt/MailScanner/etc/reports/en/inline.sig.txt
-inlinetextwarning	/opt/MailScanner/etc/reports/en/inline.warning.txt
+DeletedContentMessage	/etc/MailScanner/reports/en/deleted.content.message.txt
+DeletedFilenameMessage	/etc/MailScanner/reports/en/deleted.filename.message.txt
+DeletedVirusMessage	/etc/MailScanner/reports/en/deleted.virus.message.txt
+DisinfectedReportText	/etc/MailScanner/reports/en/disinfected.report.txt
+inlinehtmlsig		/etc/MailScanner/reports/en/inline.sig.html
+inlinehtmlwarning	/etc/MailScanner/reports/en/inline.warning.html
+inlinespamwarning	/etc/MailScanner/reports/en/inline.spam.warning.txt
+inlinetextsig		/etc/MailScanner/reports/en/inline.sig.txt
+inlinetextwarning	/etc/MailScanner/reports/en/inline.warning.txt
 languagestrings		
-recipientmcpreport	/opt/MailScanner/etc/reports/en/recipient.mcp.report.txt
-recipientspamreport	/opt/MailScanner/etc/reports/en/recipient.spam.report.txt
-sendercontentreport 	/opt/MailScanner/etc/reports/en/sender.content.report.txt
-sendererrorreport 	/opt/MailScanner/etc/reports/en/sender.error.report.txt
-senderfilenamereport	/opt/MailScanner/etc/reports/en/sender.filename.report.txt
-SenderRBLSpamReport	/opt/MailScanner/etc/reports/en/sender.spam.rbl.report.txt
-SenderSASpamReport	/opt/MailScanner/etc/reports/en/sender.spam.sa.report.txt
-SenderSAMCPReport	/opt/MailScanner/etc/reports/en/sender.mcp.report.txt
-SenderBothSpamReport	/opt/MailScanner/etc/reports/en/sender.spam.report.txt
-sendervirusreport 	/opt/MailScanner/etc/reports/en/sender.virus.report.txt
-StoredContentMessage	/opt/MailScanner/etc/reports/en/stored.content.message.txt
-StoredFilenameMessage	/opt/MailScanner/etc/reports/en/stored.filename.message.txt
-StoredVirusMessage	/opt/MailScanner/etc/reports/en/stored.virus.message.txt
+recipientmcpreport	/etc/MailScanner/reports/en/recipient.mcp.report.txt
+recipientspamreport	/etc/MailScanner/reports/en/recipient.spam.report.txt
+rejectionreport		/etc/MailScanner/reports/en/message.rejection.report.txt
+sendercontentreport 	/etc/MailScanner/reports/en/sender.content.report.txt
+sendererrorreport 	/etc/MailScanner/reports/en/sender.error.report.txt
+senderfilenamereport	/etc/MailScanner/reports/en/sender.filename.report.txt
+SenderRBLSpamReport	/etc/MailScanner/reports/en/sender.spam.rbl.report.txt
+SenderSASpamReport	/etc/MailScanner/reports/en/sender.spam.sa.report.txt
+SenderSAMCPReport	/etc/MailScanner/reports/en/sender.mcp.report.txt
+SenderBothSpamReport	/etc/MailScanner/reports/en/sender.spam.report.txt
+sendervirusreport 	/etc/MailScanner/reports/en/sender.virus.report.txt
+StoredContentMessage	/etc/MailScanner/reports/en/stored.content.message.txt
+StoredFilenameMessage	/etc/MailScanner/reports/en/stored.filename.message.txt
+StoredVirusMessage	/etc/MailScanner/reports/en/stored.virus.message.txt
 
 [First,Command]
 Sendmail		/usr/sbin/sendmail
@@ -478,7 +490,7 @@ AllowFormTags		0	no	0	yes	1	disarm	convert
 AllowObjectTags		0	no	0	yes	1	disarm	convert
 AllowScriptTags		0	no	0	yes	1	disarm	convert
 AllowPassZips		0	no	0	yes	1
-AllowWebBugTags		1	yes	1	disarm	convert
+AllowWebBugTags		1	no	0	yes	1	disarm	convert
 BlockEncrypted		0	no	0	yes	1
 BlockUnencrypted	0	no	0	yes	1
 DangerScan		1	no	0	yes	1
@@ -492,15 +504,17 @@ QuarantineInfections	1	no	0	yes	1
 QuarantineModifiedBody	0	no	0	yes	1
 QuarantineSilent	1	no	0	yes	1
 QuarantineWholeMessage	0	no	0	yes	1
+RejectMessage		0	no	0	yes	1
+ReplaceTNEF		2	no	0	add	1	replace	2
 ScanMail		1	no	0	yes	1
 SpamStars		0	no	0	yes	1
 TagPhishingSubject	0	no	0 	yes	1
 MCPUseSpamAssassin	1	no	0	yes	1
-UseSpamAssassin		0	no	0	yes	1
+UseSpamAssassin		1	no	0	yes	1
 VirusScan		1	no	0	yes	1
 
 [All,File]
-#FilenameRules		/opt/MailScanner/etc/filename.rules.conf
+#FilenameRules		/etc/MailScanner/filename.rules.conf
 
 [All,Other]
 # This is the other stuff that came up in the search that I haven't
@@ -514,4 +528,7 @@ NoticeRecipient			postmaster
 RemoveHeaders
 SilentViruses			All-Viruses
 SpamDomainList			
-
+allowfilenames
+denyfiletypes
+allowfiletypes
+denyfilenames
