@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: CustomConfig.pm,v 1.3.2.33 2005/03/16 17:00:56 jkf Exp $
+#   $Id: CustomConfig.pm,v 1.3.2.36 2006/01/14 21:56:52 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.3.2.33 $, 10;
+$VERSION = substr q$Revision: 1.3.2.36 $, 10;
 
 #
 # These are the custom functions that you can write to produce a value
@@ -767,7 +767,8 @@ my $ConfFile = $ARGV[0];
 $ConfFile = $ARGV[1] if $ConfFile =~ /^-+v/i;
 
 my ($AccessDB, $Refusal, $my_mta); # Generalized here for Multiple MTA support
-$my_mta = lc(MailScanner::Config::QuickPeek($ConfFile, 'mta')) if $ConfFile;
+$my_mta = lc(MailScanner::Config::QuickPeek($ConfFile, 'mta')) if $ConfFile &&
+                                                               -f $ConfFile;
 if ($my_mta eq "postfix") { 
   $AccessDB = "/etc/postfix/access.db";
   $Refusal = "$FailCode Site blocked by MailScanner due to excessive email";
@@ -849,7 +850,7 @@ sub InitIPBlock {
     $counter++;
   }
   close($LimitsH);
-  MailScanner::Log::InfoLog("Read %d IP blocking whitelist entries from %s",
+  MailScanner::Log::InfoLog("Read %d IP blocking entries from %s",
                             $counter, $WhitelistFile);
 }
 
