@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Exim.pm,v 1.24.2.40 2005/03/27 18:21:32 jkf Exp $
+#   $Id: Exim.pm,v 1.24.2.43 2006/01/16 10:53:35 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use IO::Pipe;
 use Carp;
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.24.2.40 $, 10;
+$VERSION = substr q$Revision: 1.24.2.43 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process
 # a message that is piped to it. Still need to add the envelope sender
@@ -415,7 +415,14 @@ sub ReadQf {
       # Add recipient to message data
       # but deal with "special" lines first
       # (when "one_time" option is being used)
+      # strips old "special" content <4.10
       $line =~ s/ \d+,\d+,\d+$//;
+      #BROKEN # strips new "special" content >= 4.10
+      #BROKEN $line =~ s/ (\d+),\d+#01$//;
+      #BROKEN if (defined $1) {
+      #BROKEN   $line = substr($line, 0, length($line)-$1-1);
+      #BROKEN }
+
       push @{$message->{to}}, $line;
     }
   }
@@ -552,6 +559,10 @@ sub ReadQf {
   # onto the metadata headers array at this point...
   exists $header->{name} and push @{$metadata{headers}},$header;
 
+  # Decode the ISO encoded Subject line
+  # Over-ride the default default character set handler so it does it
+  # much better than the MIME-tools default handling.
+  MIME::WordDecoder->default->handler('*' => \&MailScanner::Message::WordDecoderKeep7Bit);
   # Decode the ISO encoded Subject line
   $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
 
