@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: EximDiskStore.pm,v 1.14.2.9 2004/12/14 16:29:33 jkf Exp $
+#   $Id: EximDiskStore.pm,v 1.14.2.11 2005/12/20 18:19:35 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.14.2.9 $, 10;
+$VERSION = substr q$Revision: 1.14.2.11 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 1.14.2.9 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 1.14.2.9 $, 10;
+#$VERSION = substr q$Revision: 1.14.2.11 $, 10;
 
 # Attributes are
 #
@@ -474,6 +474,54 @@ sub CopyEntireMessage {
   return 1;
 }
 
+# Writes the whole message to a handle.
+# Need to be passed the message to find the headers path
+# as it's not part of the DiskStore.
+sub ReadMessageHandle {
+  my $this = shift;
+  my ($message, $handle) = @_;
+
+  my $hhandle = $message->{headerspath};
+  my $dhandle = $this->{dpath};
+
+  # File::Copy does not close our handles
+  # so locks are preserved
+  copy($hhandle , $handle);
+
+  # We have to strip the message-ID off the beginning of the file
+  # using sysread since that is what File::Copy uses and it doesn't
+  # play nicely with stdio operations such as $body->getline. The
+  # magic number 19 is from the length of NNNNNN-NNNNNN-NN-D\n.
+  my $from_h = \do { local *FH1 };
+  open($from_h, "< $dhandle");
+  binmode $from_h or die "($!,$^E)";
+
+  sysseek($from_h, 19, 0);
+
+  my $size;
+  my $buf = "";
+  $size = -s $dhandle;
+  $size = 1024 if ($size < 512);
+  $size = 1024*1024*2 if $size > 1024*1024*2;
+  local($\) = '';
+  my $to_h   = $handle;
+
+  for (;;) {
+    my ($r, $w, $t);
+    $r = sysread($from_h, $buf, $size);
+    last unless $r;
+    for ($w = 0; $w < $r; $w += $t) {
+      $t = syswrite($to_h, $buf, $r - $w, $w);
+    }
+  }
+
+  # rewind tmpfile to read it later
+  sysseek($handle, 0, 0); # Rewind the file
+
+  return 1;
+}
+
+
 
 # Produce a pipe that will read the whole message.
 # Need to be passed the message to find the headers path
