@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Log.pm,v 1.14.2.9 2005/08/07 15:20:08 jkf Exp $
+#   $Id: Log.pm,v 1.14.2.10 2006/01/14 19:24:15 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -113,7 +113,7 @@ sub InfoLog {
 
   LogText($logmessage, 'info');
 
-  print STDERR "@x\n" if $LogType eq 'stderr';
+  print STDERR "$logmessage\n" if $LogType eq 'stderr';
 }
 
 sub DebugLog {
@@ -123,7 +123,7 @@ sub DebugLog {
 
     LogText($logmessage, 'debug');
 
-    print STDERR "@x\n" if $LogType eq 'stderr';
+    print STDERR "$logmessage\n" if $LogType eq 'stderr';
   }
 }
 
