@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm,v 1.64.2.37 2005/08/28 15:08:49 jkf Exp $
+#   $Id: MessageBatch.pm,v 1.64.2.49 2006/03/02 22:45:24 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,12 +40,12 @@ use strict 'refs';
 no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
-
+use Time::HiRes qw ( time );
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.64.2.37 $, 10;
+$VERSION = substr q$Revision: 1.64.2.49 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -95,14 +95,25 @@ sub EndBatch {
   my $totaltime  = $now - $this->{starttime};
   my $speed;
 
+  #print STDERR "Started at " . $this->{starttime} . "\n";
+  #print STDERR "Finished at " . $now . "\n";
+  #print STDERR "Tool " . $totaltime . "\n";
+
   $this->{endtime} = $now;
   $this->{totaltime} = $totaltime;
-  $totaltime = 1 unless $totaltime > 0; # Minimum of 1 second
+  $totaltime = 1 unless $totaltime > 0.001; # Minimum of 1 m-second
   $speed = ($totalbytes*1.0) / ($totaltime*1.0) if $totaltime > 0;
   $this->{bytespersecond} = $speed;
   MailScanner::Log::InfoLog("Batch completed at %d bytes per second (%d / %d)",
                             $speed, $totalbytes, $now-$this->{starttime})
     if MailScanner::Config::Value('logspeed');
+
+  # Work out current size of batch.
+  my $msgs = $this->{messages};
+  my $msgcount = scalar(keys %$msgs);
+
+  MailScanner::Log::InfoLog("Batch (%d messages) processed in %.2f seconds",
+                            $msgcount, $totaltime);
 }
 
 # Start the timing for a section of the main code
@@ -233,6 +244,9 @@ sub SpamChecks {
   MailScanner::Log::InfoLog("Spam Checks: Starting")
     if MailScanner::Config::Value('logspam');
 
+  # If the cache contents have expired then clean it all up first
+  MailScanner::SA::CheckForCacheExpire();
+
   while(($id, $message) = each %{$this->{messages}}) {
     next unless $message->{scanmail};
     next if $message->{deleted};
@@ -297,6 +311,22 @@ sub HandleHam {
   #print STDERR "Finished handling ham\n\n";
 }
 
+# Reject messages that come from people we want to reject. Send nice report
+# instead.
+sub RejectMessages {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    # Skip deleted and non-spam messages
+    next if $message->{deleted};
+    #print STDERR "May reject message $id\n";
+    $message->RejectMessage()
+      if MailScanner::Config::Value('rejectmessage',$message) !~ /0/;
+  }
+}
+
 # Do all the MCP checks.
 # Must have removed deleted messages from the batch first.
 sub MCPChecks {
@@ -433,16 +463,19 @@ sub Explode {
   my($key, $message);
 
   # jjh 2004-03-12 reap as many as we can.
-  1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+  # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+  1 until waitpid(-1, WNOHANG) == -1;
 
   #print STDERR "About to explode messages\n";
+  print STDERR "Ignore errors about failing to find EOCD signature\n";
   umask $global::MS->{work}->{fileumask};
   while(($key, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
     #print STDERR "About to explode message $key $message\n";
     $message->Explode();
     # jjh 2004-03-12 reap as many as we can.
-    1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+    # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+    1 until waitpid(-1, WNOHANG) == -1;
   }
   umask 0077;
 }
@@ -1035,6 +1068,17 @@ sub Encapsulate {
   }
 }
 
+# Add the virus statistics to the SpamAssassin results cache
+# so we know to keep the cache record for much longer.
+sub AddVirusInfoToCache {
+  my $this = shift;
+
+  my($id, $message);
+  while(($id, $message) = each %{$this->{messages}}) {
+    MailScanner::SA::AddVirusStats($message);
+  }
+}
+
 
 # This simply evaluates a known configuration parameter for every message
 # in the batch, whether it has been deleted or not. This solely exists so
@@ -1043,10 +1087,21 @@ sub Encapsulate {
 sub LastLookup {
   my $this = shift;
 
+  my $start = time;
+
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
     MailScanner::Config::Value('lastlookup', $message);
   }
+
+  unless (MailScanner::Config::IsSimpleValue('lastlookup') &&
+          !MailScanner::Config::Value('lastlookup')) {
+    MailScanner::Log::InfoLog("\"Always Looked Up Last\" took %.2f seconds",
+                              time-$start+0.0);
+  }
+
+  # Lookup one remaining value after end of batch
+  MailScanner::Config::Value('lastafterbatch');
 }
 
 1;
