@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: PFDiskStore.pm,v 1.1.2.37 2005/07/05 14:14:51 jkf Exp $
+#   $Id: PFDiskStore.pm,v 1.1.2.40 2005/12/29 16:25:35 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.37 $, 10;
+$VERSION = substr q$Revision: 1.1.2.40 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 1.1.2.37 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 1.1.2.37 $, 10;
+#$VERSION = substr q$Revision: 1.1.2.40 $, 10;
 
 # Attributes are
 #
@@ -546,6 +546,57 @@ sub CopyEntireMessage {
 #  }
 #}
 
+# Writes the whole message to a handle.
+# Need to be passed the message to find the headers path
+# as it's not part of the DiskStore.
+sub ReadMessageHandle {
+  my $this = shift;
+  my ($message, $handle) = @_;
+
+  # we use already opened handles
+  my $hdhandle = $this->{inhdhandle};
+
+  # Where did we start?
+  my $oldpos = sysseek $hdhandle, 0, 1;
+  #print STDERR "Old position = $oldpos\n";
+
+  # rewind files to read and write with File::Copy
+  sysseek($hdhandle, 0, 0) or die "$!,$^E"; # Rewind the file
+
+  # Write the whole message in RFC822 format to the handle.
+  # That means 1 CR-terminated line for every N record in the file.
+  my $b = Body->new($hdhandle);
+  #if ($b) {
+    $b->Start(1); # 1 says we want the headers as well as the body
+    my $line;
+    #print STDERR "\n\n\n\n\n";
+    while(defined($line = $b->Next())) {
+      #print STDERR "print $line\n";
+      print $handle "$line\n" or die "$!, $^E";
+    }
+    $b->Done();
+  #} else {
+  #  die "Couldn't create new body object from $hdhandle, $!, $^E";
+  #}
+
+  # rewind tmpfile to read it later
+  $handle->seek(0,0) or die "$!, $^E"; # Rewind the file
+  #print STDERR "\n\n\nTmp File is this:\n";
+  #while(<$handle>) {
+  #  print STDERR $_;
+  #}
+  #print STDERR "Tmp File End\n";
+  #$handle->seek(0,0) or die "$!, $^E"; # Rewind the file
+
+  # rewind source files
+  sysseek($hdhandle, 0, 0); # Rewind the file
+  sysseek($hdhandle, $oldpos, 0); # Rewind the file
+
+  #print STDERR "Done ReadMessageHandle\n";
+  return 1;
+}
+
+
 
 # Produce a pipe that will read the whole message.
 # Need to be passed the message to find the headers path
