@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm,v 1.1.2.64 2005/07/31 16:28:18 jkf Exp $
+#   $Id: Postfix.pm,v 1.1.2.66 2005/12/07 14:00:49 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.64 $, 10;
+$VERSION = substr q$Revision: 1.1.2.66 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -307,19 +307,28 @@ sub new {
     #print "1st $rectype is \"$recdata\"\n";
     MailScanner::Log::WarnLog("Syntax error in Postfix queue file, didn't " .
                               "start with a C record") unless $rectype eq 'C';
-    $recdata =~ /^([0-9 ]{15}) ([0-9 ]{15}) ([0-9 ]{15})( ([0-9 ]{15}))?$/;
-    ($MsgContSize, $DataOffset, $NumRecips) = ($1+0, $2+0, $3+0);
+    #$recdata =~ /^([0-9 ]{15}) ([0-9 ]{15}) ([0-9 ]{15})( ([0-9 ]{15}))?$/;
+    #($MsgContSize, $DataOffset, $NumRecips) = ($1+0, $2+0, $3+0);
+
+    my @numbers = split " ", $recdata;
+    ($MsgContSize, $DataOffset, $NumRecips) =
+      ($numbers[0]+0, $numbers[1]+0, $numbers[2]+0);
 
     # If $5 is set then we have a new data structure in the file
     $MailScanner::Postfix::DataStructure = 0;
-    if ($5 ne "") {
+    #if ($5 ne "") {
+    #  $MailScanner::Postfix::DataStructure = 1;
+    #  $message->{PostfixQmgrOpts} = $5+0;
+    #}
+
+    if (defined $numbers[3]) {
       $MailScanner::Postfix::DataStructure = 1;
-      $message->{PostfixQmgrOpts} = $5+0;
+      $message->{PostfixQmgrOpts} = $numbers[3]+0;
     }
 
-    $MsgContSize =~ s/^\s*//;
-    $DataOffset  =~ s/^\s*//;
-    $NumRecips   =~ s/^\s*//;
+    #$MsgContSize =~ s/^\s*//;
+    #$DataOffset  =~ s/^\s*//;
+    #$NumRecips   =~ s/^\s*//;
     #print STDERR "MsgContSize=$MsgContSize DataOffset=$DataOffset NumRecips=$NumRecips\n";
     push @{$message->{metadata}}, "$rectype$recdata";
     #print STDERR "Content size = $MsgContSize\n";
@@ -329,7 +338,12 @@ sub new {
     # If the data offset is 0 then Postfix definitely hasn't finished
     # writing the message.
     unless ($DataOffset+0 > 10) { # 10 == arbitrary small number
-      $message->DropFromBatch();
+      # JKF 5/12/2005 This could fail with an unblessed reference error
+      # JKF 5/12/2005 so do it by hand.
+      # JKF 5/12/2005 $message->DropFromBatch();
+      $message->{deleted} = 1;
+      $message->{gonefromdisk} = 1; # Don't try to delete the original
+      $message->{store}->Unlock(); # Unlock it so other processes can pick it up
       return 0;
     }
 
@@ -491,6 +505,9 @@ sub new {
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
     # Decode the ISO encoded Subject line
+    # Over-ride the default default character set handler so it does it
+    # much better than the MIME-tools default handling.
+    MIME::WordDecoder->default->handler('*' => \&MailScanner::Message::WordDecoderKeep7Bit);
     $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
 
     # If we never found the E (end of message file) record, then
@@ -498,7 +515,13 @@ sub new {
     unless ($ERecordFound) {
       MailScanner::Log::WarnLog("No end-of-message record found in %s, " .
                                 "retrying", $message->{id});
-      $message->DropFromBatch();
+      # JKF 5/12/2005 This could fail with an unblessed reference error
+      # JKF 5/12/2005 so do it by hand.
+      # JKF 5/12/2005 $message->DropFromBatch();
+      $message->{deleted} = 1;
+      $message->{gonefromdisk} = 1; # Don't try to delete the original
+      $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+
       return 0;
     }
 
