@@ -5,7 +5,7 @@
 #   OpenProtect - Server Side E-Mail Protection	
 #   Copyright (C) 2003 Opencomputing Technologies
 #
-#   $Id: Qmail.pm,v 1.1.2.9 2005/03/27 18:21:32 jkf Exp $
+#   $Id: Qmail.pm,v 1.1.2.10 2005/11/20 14:53:42 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.9 $, 10;
+$VERSION = substr q$Revision: 1.1.2.10 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -271,6 +271,9 @@ sub new {
     return 1 if $TOFound;
     
     # Decode the ISO encoded Subject line
+    # Over-ride the default default character set handler so it does it
+    # much better than the MIME-tools default handling.
+    MIME::WordDecoder->default->handler('*' => \&MailScanner::Message::WordDecoderKeep7Bit);
     $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
 
     $message->{store}->DeleteUnlock();
