@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm,v 1.35.2.53 2005/09/15 10:27:50 jkf Exp $
+#   $Id: SA.pm,v 1.35.2.72 2006/02/01 22:19:57 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -38,13 +38,13 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 
 use IO;
 use POSIX qw(:signal_h); # For Solaris 9 SIG bug workaround
-# Don't do this any more as SpamAssassin prefers to do it itself
-# use AnyDBM_File; # Doing this here keeps SpamAssassin quiet
+use DBI;
+use Compress::Zlib;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.35.2.53 $, 10;
+$VERSION = substr q$Revision: 1.35.2.72 $, 10;
 
 # Attributes are
 #
@@ -59,10 +59,18 @@ my $SAversion;           # SpamAssassin version number
 my @SAsuccessqueue;      # queue of failure history
 my $SAsuccessqsum;       # current sum of history queue
 
-my($SAspamtest, $SABayesLock, $SABayesRebuildLock);
+my($SAspamtest, $SABayesLock, $SABayesRebuildLock, $SpamAssassinInstalled);
+
+my($SQLiteInstalled, $cachedbh, $cachefilename, $NextCacheExpire);
+
+my $HamCacheLife      = 30*60;    # Lifetime of non-spam from first seen
+my $SpamCacheLife     = 5*60;     # Lifetime of low-scoring spam from first seen
+my $HighSpamCacheLife = 3*60*60;  # Lifetime of high spam from last seen
+my $VirusesCacheLife  = 48*60*60; # Lifetime of viruses from last seen
+my $ExpireFrequency   = 10*60;    # How often to run the expiry of the cache
 
 sub initialise {
-  my($RebuildBayes) = @_; # Start by rebuilding the Bayes database?
+  my($RebuildBayes, $WantLintOnly) = @_; # Start by rebuilding the Bayes database?
 
   my(%settings, $val, $val2, $prefs);
 
@@ -86,9 +94,10 @@ sub initialise {
   unless (MailScanner::Config::IsSimpleValue('usespamassassin') &&
           !MailScanner::Config::Value('usespamassassin')) {
     $settings{dont_copy_prefs} = 1; # Removes need for home directory
-    $prefs = MailScanner::Config::Value('spamassassinprefsfile');
-    $settings{userprefs_filename} = $prefs if defined $prefs;
-    $val = MailScanner::Config::Value('debugspamassassin');
+    # This file is now read directly by SpamAssassin's normal startup code.
+    #$prefs = MailScanner::Config::Value('spamassassinprefsfile');
+    #$settings{userprefs_filename} = $prefs if defined $prefs;
+    $val = $MailScanner::SA::Debug;
     $settings{debug} = $val;
     # for unusual bayes and auto whitelist database locations
     $val = MailScanner::Config::Value('spamassassinuserstatedir');
@@ -116,13 +125,81 @@ sub initialise {
       unshift @INC, "$val/lib/perl5/site_perl/$perl_vers";
     }
     # Now we have the path built, try to find the SpamAssassin modules
-    MailScanner::Log::DieLog("SpamAssassin installation could not be found")
-      unless eval "require Mail::SpamAssassin";
+    unless (eval "require Mail::SpamAssassin") {
+      MailScanner::Log::WarnLog("You want to use SpamAssassin but have not installed it.");
+      MailScanner::Log::WarnLog("Please download http://www.sng.ecs.soton.ac.uk/mailscanner/files/4/install-Clam-SA.tar.gz and unpack it and run ./install.sh to install it, then restart MailScanner.");
+      MailScanner::Log::WarnLog("I will run without SpamAssassin for now, you will not detect much spam until you install SpamAssassin.");
+      $SpamAssassinInstalled = 0;
+      return;
+    }
+
+    # SpamAssassin "require"d okay.
+    $SpamAssassinInstalled = 1;
+
     # Find the version number
     $SAversion = $Mail::SpamAssassin::VERSION + 0.0;
 
+    #
+    # Load the SQLite support for the SA data cache
+    #
+    $SQLiteInstalled = 0;
+    unless (MailScanner::Config::IsSimpleValue('usesacache') &&
+            !MailScanner::Config::Value('usesacache')) {
+      unless (eval "require DBD::SQLite") {
+        MailScanner::Log::WarnLog("WARNING: You are trying to use the SpamAssassin cache but your DBI and/or DBD::SQLite Perl modules are not properly installed!");
+        $SQLiteInstalled = 0;
+      } else {
+        $SQLiteInstalled = 1;
+        unless (eval "require Digest::MD5") {
+          MailScanner::Log::WarnLog("WARNING: You are trying to use the SpamAssassin cache but your Digest::MD5 Perl module is not properly installed!");
+          $SQLiteInstalled = 0;
+        } else {
+          MailScanner::Log::InfoLog("Using SpamAssassin results cache");
+          $SQLiteInstalled = 1;
+          #
+          #
+          # Put the SA cache database initialisation code here!
+          #
+          #
+          $MailScanner::SA::cachefilename = MailScanner::Config::Value("sacache");
+          $MailScanner::SA::cachedbh = DBI->connect(
+                                  "dbi:SQLite:$MailScanner::SA::cachefilename",
+                                  "","",{PrintError=>0,InactiveDestroy=>1});
+          $NextCacheExpire = $ExpireFrequency+time;
+          if ($MailScanner::SA::cachedbh) {
+            MailScanner::Log::InfoLog("Connected to SpamAssassin cache database");
+            # Rebuild all the tables and indexes. The PrintError=>0 will make it
+            # fail quietly if they already exist.
+            $MailScanner::SA::cachedbh->do("CREATE TABLE cache (md5 TEXT, count INTEGER, last TIMESTAMP, first TIMESTAMP, sasaysspam INT, sahighscoring INT, sascore FLOAT, saheader BLOB, salongreport BLOB, virusinfected INT)");
+            $MailScanner::SA::cachedbh->do("CREATE UNIQUE INDEX md5_uniq ON cache(md5)");
+            $MailScanner::SA::cachedbh->do("CREATE INDEX last_seen_idx ON cache(last)");
+            $MailScanner::SA::cachedbh->do("CREATE INDEX first_seen_idx ON cache(first)");
+            $SQLiteInstalled = 1;
+            SetCacheTimes();
+            # Now expire all the old tokens
+            CacheExpire() unless $WantLintOnly;
+          } else {
+            MailScanner::Log::WarnLog("Could not create SpamAssassin cache database %s", $MailScanner::SA::cachefilename);
+            $SQLiteInstalled = 0;
+            print STDERR "Could not create SpamAssassin cache database $MailScanner::SA::cachefilename\n" if $WantLintOnly;
+          }
+        }
+      }
+    }
+
     $MailScanner::SA::SAspamtest = new Mail::SpamAssassin(\%settings);
 
+    if ($WantLintOnly) {
+      my $errors = $MailScanner::SA::SAspamtest->lint_rules();
+      if ($errors) {
+        print STDERR "SpamAssassin reported an error.\n";
+        $MailScanner::SA::SAspamtest->debug_diagnostics();
+      } else {
+        print STDERR "SpamAssassin reported no errors.\n";
+      }
+      return;
+    }
+
     # Rebuild the Bayes database if it is due
     $MailScanner::SA::BayesRebuildLock = MailScanner::Config::Value(
                           'lockfiledir') . '/MS.bayes.rebuild.lock';
@@ -148,6 +225,7 @@ sub initialise {
                        $MailScanner::SA::BayesRebuildLock, $!);
 
         # Do the actual expiry run
+        $0 = 'MailScanner: rebuilding Bayes database';
         MailScanner::Log::InfoLog('SpamAssassin Bayes database rebuild starting');
         eval {
           $MailScanner::SA::SAspamtest->init(1) if $SAversion<3;
@@ -197,10 +275,10 @@ sub initialise {
       my $t = $MailScanner::SA::SAspamtest;
       $MailScanner::SA::SABayesLock = $t->sed_path($t->{conf}->{bayes_path}) .
                                       '.lock';
-      print STDERR "SA bayes lock is $MailScanner::SA::SABayesLock\n";
+      #print STDERR "SA bayes lock is $MailScanner::SA::SABayesLock\n";
     };
 
-    print STDERR "Bayes lock is at $MailScanner::SA::SABayesLock\n";
+    #print STDERR "Bayes lock is at $MailScanner::SA::SABayesLock\n";
     # JKF 7/1/2002 Commented out due to it causing false positives
     # JKF 7/6/2002 Now has a config switch
     # JKF 12/6/2002 Remember to read the prefs file
@@ -234,6 +312,23 @@ sub initialise {
   $^W = 0;
 }
 
+# Set all the cache expiry timings from the cachetiming conf option
+sub SetCacheTimes {
+  my $line = MailScanner::Config::Value('cachetiming');
+  $line =~ s/^\D+//;
+  return unless $line;
+  my @numbers = split /\D+/, $line;
+  return unless @numbers;
+
+  $HamCacheLife      = $numbers[0] if $numbers[0];
+  $SpamCacheLife     = $numbers[1] if $numbers[1];
+  $HighSpamCacheLife = $numbers[2] if $numbers[2];
+  $VirusesCacheLife  = $numbers[3] if $numbers[3];
+  $ExpireFrequency   = $numbers[4] if $numbers[4];
+
+  #print STDERR "Timings are \"" . join(' ',@numbers) . "\"\n";
+}
+
 
 # Constructor.
 sub new {
@@ -248,6 +343,9 @@ sub new {
 sub Checks {
   my $message = shift;
 
+  # If they never actually installed SpamAssassin, then just bail out quietly.
+  return (0,0,"",0,"") unless $SpamAssassinInstalled;
+
   my($dfhandle);
   my($dfilename, $dfile, @WholeMessage, $SAResult, $SAHitList);
   my($HighScoring, $SAScore, $maxsize, $SAReport, $GSHits);
@@ -288,6 +386,7 @@ sub Checks {
     if (-e $MailScanner::SA::BayesRebuildStartLock) {
       # Do we wait for Bayes rebuild to occur?
       if ($MailScanner::SA::WaitForRebuild) {
+        $0 = 'MailScanner: waiting for Bayes rebuild';
         # Wait quietly for the file to disappear
         # This must not take more than 1 hour or we are in trouble!
         #MailScanner::Log::WarnLog("Waiting for rebuild start request to disappear");
@@ -300,6 +399,7 @@ sub Checks {
         # Did it take too long?
         unlink $MailScanner::SA::BayesRebuildStartLock if $waiter>=3590;
         #MailScanner::Log::WarnLog("Start request has disappeared");
+        $0 = 'MailScanner: checking with SpamAssassin';
       } else {
         # Return saying we are skipping SpamAssassin this time
         return (0,0, 'SpamAssassin rebuilding', 0);
@@ -381,49 +481,219 @@ sub Checks {
   }
 
   push(@WholeMessage, "\n");
-  $message->{store}->ReadBody(\@WholeMessage, $maxsize);
 
-  #print STDERR "Whole message is this:\n";
-  #print STDERR "----------------------\n";
-  #print STDERR @WholeMessage;
-  #print STDERR "---------------\n";
-  #print STDERR "End of message.\n";
+  my(@WholeBody);
+  $message->{store}->ReadBody(\@WholeBody, $maxsize);
+  push(@WholeMessage, @WholeBody);
+
+  # Work out the MD5 sum of the body
+  my($testcache,$md5,$md5digest);
+  if ($SQLiteInstalled) {
+    $testcache = MailScanner::Config::Value("usesacache",$message);
+    $testcache = ($testcache =~ /1/)?1:0;
+    $md5 = Digest::MD5->new;
+    #foreach(@WholeBody) {
+    #  $md5->add($_);
+    #}
+    eval { $md5->add(@WholeBody) };
+    if ($@ ne "" || @WholeBody<=1) {
+      # The eval failed
+      $md5digest = "unknown";
+      $testcache = 0;
+    } else {
+      # The md5->add worked okay, so use the results
+      # Get the MD5 digest of the message body
+      $md5digest = $md5->hexdigest;
+    }
+
+    # Store it for later
+    $message->{md5} = $md5digest;
+    #print STDERR "MD5 digest is $md5digest\n";
+  } else {
+    $testcache = 0;
+    #print STDERR "Not going to use cache\n";
+  }
 
   # Now construct the SpamAssassin object for version < 3
   my $spammail;
   $spammail = Mail::SpamAssassin::NoMailAudit->new('data'=>\@WholeMessage)
     if $SAversion<3;
 
-  #print STDERR "NoMailAudit thinks the message is this:\n";
-  #print STDERR "---------------------------------------\n";
-  #print STDERR $spammail->as_string();
-  #print STDERR "---------------\n";
-  #print STDERR "End of message.\n";
+  if ($testcache) {
+    if (my $cachehash = CheckCache($md5digest)) {
+      #print STDERR "Cache hit for " . $message->{id} . "\n";
+      MailScanner::Log::InfoLog("SpamAssassin cache hit for message %s", $message->{id});
+      # Read the cache result and update the timestamp *****
+      ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) =
+      ($cachehash->{sasaysspam}, $cachehash->{sahighscoring},
+       uncompress($cachehash->{saheader}),   $cachehash->{sascore},
+       uncompress($cachehash->{salongreport}));
 
-  # Test it for spam-ness
-  if ($SAversion<3) {
-    ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
-      = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
-                      $spammail, $message);
+      #print STDERR "Cache results are $SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport\n";
+      # Unlock and close the lockfile
+      flock($RebuildLockH, $LOCK_UN) if $MailScanner::SA::DoingBayesRebuilds; # $BayesIsLocked;
+      $RebuildLockH->close() if $MailScanner::SA::DoingBayesRebuilds;
+    } else {
+      # Do the actual SpamAssassin call
+      #print STDERR "Cache miss for " . $message->{id} . "\n";
+
+      # Test it for spam-ness
+      if ($SAversion<3) {
+        ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
+          = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
+                          $spammail, $message);
+      } else {
+        #print STDERR "Check 1, report template = \"" .
+        #      $MailScanner::SA::SAspamtest->{conf}->{report_template} . "\"\n";
+        ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
+          = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
+                          \@WholeMessage, $message);
+      }
+
+      #MailScanner::Log::WarnLog("Done SAForkAndTest");
+      #print STDERR "SAResult = $SAResult\nHighScoring = $HighScoring\n" .
+      #             "SAHitList = $SAHitList\n";
+
+      # Write the record to the cache *****
+      CacheResult($md5digest, $SAResult, $HighScoring, compress($SAHitList), $SAScore, compress($SAReport));
+
+      # Unlock and close the lockfile
+      flock($RebuildLockH, $LOCK_UN) if $MailScanner::SA::DoingBayesRebuilds; # $BayesIsLocked;
+      $RebuildLockH->close() if $MailScanner::SA::DoingBayesRebuilds;
+    }
   } else {
-    #print STDERR "Check 1, report template = \"" .
-    #      $MailScanner::SA::SAspamtest->{conf}->{report_template} . "\"\n";
-    ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
-      = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
-                      \@WholeMessage, $message);
+    # No cache here
+
+    # Test it for spam-ness
+    if ($SAversion<3) {
+      ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
+        = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
+                        $spammail, $message);
+    } else {
+      #print STDERR "Check 1, report template = \"" .
+      #      $MailScanner::SA::SAspamtest->{conf}->{report_template} . "\"\n";
+      ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) 
+        = SAForkAndTest($GSHits, $MailScanner::SA::SAspamtest,
+                        \@WholeMessage, $message);
+    }
+    #MailScanner::Log::WarnLog("Done SAForkAndTest");
+    #print STDERR "SAResult = $SAResult\nHighScoring = $HighScoring\n" .
+    #             "SAHitList = $SAHitList\n";
+    # Unlock and close the lockfile
+    flock($RebuildLockH, $LOCK_UN) if $MailScanner::SA::DoingBayesRebuilds; # $BayesIsLocked;
+    $RebuildLockH->close() if $MailScanner::SA::DoingBayesRebuilds;
   }
 
-  #MailScanner::Log::WarnLog("Done SAForkAndTest");
-  #print STDERR "SAResult = $SAResult\nHighScoring = $HighScoring\n" .
-  #             "SAHitList = $SAHitList\n";
-
-  # Unlock and close the lockfile
-  flock($RebuildLockH, $LOCK_UN) if $MailScanner::SA::DoingBayesRebuilds; # $BayesIsLocked;
-  $RebuildLockH->close() if $MailScanner::SA::DoingBayesRebuilds;
 
   return ($SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport);
 }
 
+# Look up the passed MD5 in the cache database and return true/false
+sub CheckCache {
+ my $md5 = shift;
+
+ my($sql, $sth);
+ $sql = "SELECT md5, count, last, first, sasaysspam, sahighscoring, sascore, saheader, salongreport FROM cache WHERE md5=?";
+ my $hash = $MailScanner::SA::cachedbh->selectrow_hashref($sql,undef,$md5);
+
+ if (defined($hash)) {
+  # Cache hit!
+  #print STDERR "Cache hit $hash!\n";
+  # Update the counter and timestamp
+  $sql = "UPDATE cache SET count=count+1, last=strftime('%s','now') WHERE md5=?";
+  $sth = $MailScanner::SA::cachedbh->prepare($sql);
+  $sth->execute($md5);
+  return $hash;
+ } else {
+  # Cache miss... we'll create the cache record after SpamAssassin has run.
+  #print STDERR "Cache miss!\n";
+  return undef;
+ }
+}
+
+# Check to see if the cache should have an expiry run done, do it if so.
+sub CheckForCacheExpire {
+  # Check to see if a cache expiry run is needed
+  CacheExpire() if $NextCacheExpire<=time;
+  # NextCacheExpire is updated by CacheExpire() so not needed here.
+}
+
+sub CacheResult {
+ my ($md5, $SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport) = @_;
+
+ my $dbh = $MailScanner::SA::cachedbh;
+#print STDERR "dbh is $dbh and cachedbh is $MailScanner::SA::cachedbh\n";
+
+ my $sql = "INSERT INTO cache (md5, count, last, first, sasaysspam, sahighscoring, sascore, saheader, salongreport) VALUES (?,?,?,?,?,?,?,?,?)";
+ my $sth = $dbh->prepare($sql);
+ #print STDERR "$sth, $@\n";
+ my $now = time;
+ $sth->execute($md5,1,$now,$now,
+               $SAResult, $HighScoring, $SAScore, $SAHitList, $SAReport);
+}
+
+# Expire records from the cache database
+sub CacheExpire {
+  my $expire1 = shift || $HamCacheLife;  # non-spam
+  my $expire2 = shift || $SpamCacheLife; # low-scoring spam
+  my $expire3 = shift || $HighSpamCacheLife; # everything else except viruses
+  my $expire4 = shift || $VirusesCacheLife; # viruses
+
+  return unless $SQLiteInstalled;
+
+  my $sth = $MailScanner::SA::cachedbh->prepare("
+   DELETE FROM cache WHERE (
+   (sasaysspam=0 AND virusinfected<1 AND first<=(strftime('%s','now')-?)) OR
+   (sasaysspam>0 AND sahighscoring=0 AND virusinfected<1 AND first<=(strftime('%s','now')-?)) OR
+   (sasaysspam>0 AND sahighscoring>0 AND virusinfected<1 AND last<=(strftime('%s','now')-?)) OR
+   (virusinfected>=1 AND last<=(strftime('%s','now')-?))
+  )");
+  MailScanner::Log::DieLog("Database complained about this: %s. I suggest you delete your %s file and let me re-create it for you", $DBI::errstr, MailScanner::Config::Value("sacache")) unless $sth;
+  my $rows = $sth->execute($expire1, $expire2, $expire3, $expire4);
+  $sth->finish;
+
+  MailScanner::Log::InfoLog("Expired %s records from the SpamAssassin cache", $rows) if $rows>0;
+
+  # This is when we should do our next cache expiry (20 minutes from now)
+  $NextCacheExpire = time + $ExpireFrequency;
+}
+
+# Add the virus information to the cache entry so we can keep infected
+# attachment details a lot longer than normal spam.
+sub AddVirusStats {
+ my($message) = @_;
+ #my $virus;
+ return unless $message;
+
+ return unless $SQLiteInstalled &&
+               MailScanner::Config::Value("usesacache",$message) =~ /1/;
+
+ my $sth = $MailScanner::SA::cachedbh->prepare('UPDATE cache SET virusinfected=? WHERE md5=?');
+  
+ ## Also print 1 line for each report about this message. These lines
+ ## contain all the info above, + the attachment filename and text of
+ ## each report.
+ #my($file, $text, @report_array);
+ #while(($file, $text) = each %{$message->{allreports}}) {
+ # $file = "the entire message" if $file eq "";
+ # # Use the sanitised filename to avoid problems caused by people forcing
+ # # logging of attachment filenames which contain nasty SQL instructions.
+ # $file = $message->{file2safefile}{$file} or $file;
+ # $text =~ s/\n/ /;  # Make sure text report only contains 1 line
+ # $text =~ s/\t/ /; # and no tab characters
+ # push (@report_array, $text);
+ #}
+ #
+ #my $reports = join(",",@report_array);
+ ## This regexp only works for clamav
+ #if ($reports =~ /(.+) contains (\S+)/) { $virus = $2; }
+ 
+ $sth->execute($message->{virusinfected}, 
+               $message->{md5}) or MailScanner::Log::WarnLog($DBI::errstr);
+}
+
+
+
 # Fork and test with SpamAssassin. This implements a timeout on the execution
 # of the SpamAssassin checks, which occasionally take a *very* long time to
 # terminate due to regular expression backtracking and other nasties.
@@ -662,4 +932,89 @@ sub SAForkAndTest {
   return ($IsItSpam, $IsItHighScore, $SAHitList, $SAHits, $SAReport);
 }
 
+sub SATest {
+  my($GSHits, $Test, $Mail, $Message) = @_;
+
+  my($SAHitList, $SAHits, $SAReqHits, $IsItSpam, $IsItHighScore, $AutoLearn);
+  my($HighScoreVal, $pid2delete, $IncludeScores, $SAReport, $queuelength);
+  my $PipeReturn = 0;
+
+  $IncludeScores = MailScanner::Config::Value('listsascores', $Message);
+  $queuelength = MailScanner::Config::Value('satimeoutlen', $Message);
+
+  my($spamness, $SAResult, $HitList, @HitNames, $Hit);
+  # Do the actual tests and work out the integer result
+  if ($SAversion<3) {
+    $spamness = $Test->check($Mail);
+  } else {
+    my $mail = $Test->parse($Mail, 1);
+    $spamness = $Test->check($mail);
+  }
+  # 1st output is get_hits or get_score \n
+  $SAHits = ($SAversion<3?$spamness->get_hits():$spamness->get_score()) + 0.0;
+  $HitList  = $spamness->get_names_of_tests_hit();
+  if ($IncludeScores) {
+    @HitNames = split(/\s*,\s*/, $HitList);
+    $HitList  = "";
+    foreach $Hit (@HitNames) {
+      $HitList .= ($HitList?', ':'') . $Hit . ' ' .
+                  sprintf("%1.2f", $spamness->{conf}->{scores}->{$Hit});
+    }
+  }
+  # Get the autolearn status
+  if ($SAversion<3) {
+    # Old code
+    if (!defined $spamness->{auto_learn_status}) {
+      $AutoLearn = "no";
+    } elsif ($spamness->{auto_learn_status}) {
+      $AutoLearn = "spam";
+    } else {
+      $AutoLearn = "not spam";
+    }
+  } else {
+    # New code
+    $spamness->learn();
+    $AutoLearn = $spamness->{auto_learn_status};
+    $AutoLearn = 'no' if $AutoLearn eq 'failed' || $AutoLearn eq "";
+    $AutoLearn = 'not spam' if $AutoLearn eq 'ham';
+  }
+  # 3rd output is $HitList \n
+  $SAHitList = $HitList;
+  # JKF New code here to print out the full spam report
+  $HitList = $spamness->get_report();
+  $HitList =~ tr/\n/\0/;
+  # 4th output is $HitList \n which is now full spam report
+  $SAReport = $HitList . "\n";
+  $spamness->finish();
+
+  #print STDERR "Read SAHits = $SAHits " . scalar(localtime) . "\n";
+
+  # Construct the hit-list including the score we got.
+  my($longHitList);
+  $SAReqHits = MailScanner::Config::Value('reqspamassassinscore',$Message)+0.0;
+  $longHitList = MailScanner::Config::LanguageValue($Message, 'score') . '=' .
+                 ($SAHits+0.0) . ', ' .
+                 MailScanner::Config::LanguageValue($Message, 'required') .' ' .
+                 $SAReqHits;
+  $longHitList .= ", autolearn=$AutoLearn" unless $AutoLearn eq 'no';
+  $longHitList .= ", $SAHitList" if $SAHitList;
+  $SAHitList = $longHitList;
+
+  # SpamAssassin is known to play with the umask
+  umask 0077; # Safety net
+
+  if ($SAHits && ($SAHits+$GSHits>=$SAReqHits)) {
+    $IsItSpam = 1;
+  } else {
+    $IsItSpam = 0;
+  }
+  $HighScoreVal = MailScanner::Config::Value('highspamassassinscore',$Message);
+  if ($SAHits && $HighScoreVal>0 && ($SAHits+$GSHits>=$HighScoreVal)) {
+    $IsItHighScore = 1;
+  } else {
+    $IsItHighScore = 0;
+  }
+  return ($IsItSpam, $IsItHighScore, $SAHitList, $SAHits, $SAReport);
+}
+
 1;
