@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Sendmail.pm,v 1.56.2.34 2005/04/06 17:16:05 jkf Exp $
+#   $Id: Sendmail.pm,v 1.56.2.38 2006/02/21 19:28:33 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.56.2.34 $, 10;
+$VERSION = substr q$Revision: 1.56.2.38 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -86,8 +86,12 @@ sub new {
   $this->{TFileRegexp} = '^tf([-\\w]*)$';
   $this->{QueueFileRegexp} = '^..([-\\w]*)$';
 
-  # Patch by Kevin Spicer to detect HASFLOCK
+  # JKF 2006-01-23 Changed default to posix as this probably is now what
+  #                most systems (particularly new ones) have.
+  $this->{LockType} = "posix";
   $this->{LockType} = "flock";
+  # Patch by Kevin Spicer to detect HASFLOCK
+  # JKF -- Does not work as old sendmail versions don't say HASFLOCK but do.
   # Automatically detect locking type
   #my $cmd = MailScanner::Config::Value('sendmail') . " -bt -d0.10 < /dev/null";
   #if ( grep /HASFLOCK/, `$cmd` ) {
@@ -291,6 +295,7 @@ my($sed) = "/bin/sed";
       chomp; # Chomp everything now. We can easily add it back later.
       s/\015/ /g; # Sanitise everything by removing all embedded <CR>s
       s/\0//g;    # Remove all null bytes
+      # Doesn't work: next if /^\s*$/; # Skip blank lines inserted by fetchmail somehow
       # JKF: You can get ASCII 13 (decimal) characters in the headers, which
       # can be used to embed attachments in the headers. Remember that in
       # most Unix environments, \n = ASCII 10 (decimal).
@@ -356,7 +361,12 @@ my($sed) = "/bin/sed";
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
     # Decode the ISO encoded Subject line
+    #print STDERR "Before decode subject is \"" . $message->{subject} . "\"\n";
+    # Over-ride the default default character set handler so it does it
+    # much better than the MIME-tools default handling.
+    MIME::WordDecoder->default->handler('*' => \&MailScanner::Message::WordDecoderKeep7Bit);
     $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
+    #print STDERR "After decode subject is \"" . $message->{subject} . "\"\n";
 
     # Every qf file should at least define the sender, 1 recipient and the
     # IP address. Everything else is optional, and is preserved as
@@ -725,7 +735,7 @@ my($sed) = "/bin/sed";
     my($fh);
 
     #print STDERR '|' . MailScanner::Config::Value('sendmail', $message) .
-    #          ' ' . $SendmailOptions . '-f ' . "'$sender'" . "\n";
+    #          ' ' . $SendmailOptions . "'$sender'" . "\n";
     $fh = new FileHandle;
     $fh->open('|' . MailScanner::Config::Value('sendmail', $message) .
               " $SendmailOptions '" . $sender . "'")
@@ -749,7 +759,7 @@ my($sed) = "/bin/sed";
     my($fh);
 
     #print STDERR  '|' . MailScanner::Config::Value('sendmail', $message) .
-    #          ' ' . $SendmailOptions . '-f ' . $sender . "\n";
+    #          ' ' . $SendmailOptions . $sender . "\n";
     $fh = new FileHandle;
     $fh->open('|' . MailScanner::Config::Value('sendmail', $message) .
               " $SendmailOptions '" . $sender . "'")
