@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepContent.pm,v 1.17.2.24 2005/05/04 15:56:34 jkf Exp $
+#   $Id: SweepContent.pm,v 1.17.2.29 2006/02/21 19:28:33 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.17.2.24 $, 10;
+$VERSION = substr q$Revision: 1.17.2.29 $, 10;
 
 # Attributes are
 #
@@ -79,6 +79,7 @@ sub ScanBatch {
   $counter = 0;
   $stripcounter = 0; # No. of messages we need to strip HTML from
 
+  $0 = 'MailScanner: dangerous content scanning';
   while(($id, $message) = each %{$batch->{messages}}) {
     next if $message->{deleted};
 
@@ -143,6 +144,7 @@ sub ScanBatch {
     my $formvalue   = MailScanner::Config::Value('allowformtags',   $message);
     my $scriptvalue = MailScanner::Config::Value('allowscripttags', $message);
     my $webbugvalue = MailScanner::Config::Value('allowwebbugtags', $message);
+    my $phishingvalue = MailScanner::Config::Value('findphishing',  $message);
     my $allowiframes = 0;
     my $allowobjects = 0;
     my $allowforms   = 0;
@@ -153,12 +155,14 @@ sub ScanBatch {
     my $convertforms   = 0;
     my $convertscripts = 0;
     my $convertwebbugs = 0;
+    my $allowphishing = 0;
     # Allow the tags only if everyone allows them
     $allowiframes = 1 if $iframevalue =~ /^[1\s]+$/;
     $allowobjects = 1 if $objectvalue =~ /^[1\s]+$/;
     $allowforms   = 1 if $formvalue   =~ /^[1\s]+$/;
     $allowscripts = 1 if $scriptvalue =~ /^[1\s]+$/;
     $allowwebbugs = 1 if $webbugvalue =~ /^[1\s]+$/;
+    $allowphishing =1 if $phishingvalue !~ /1/;
     # Convert the tags if no-one blocks them and someone converts them
     $convertiframes = 1 if $iframevalue !~ /0/ && $iframevalue =~ /convert/i;
     $convertobjects = 1 if $objectvalue !~ /0/ && $objectvalue =~ /convert/i;
@@ -170,10 +174,12 @@ sub ScanBatch {
     #print STDERR "WebBugvalue = $webbugvalue\n";
     #print STDERR "Allowforms = $allowforms and convertforms = $convertforms\n";
 
+    #print STDERR "Allowphishing = $allowphishing\n";
+
     # Shortcut the check completely if they want to allow everything
     # and are not converting nasty tags to text
     if (!($allowiframes && $allowforms && $allowscripts && $allowobjects &&
-          !$stripdangerous) &&
+          $allowphishing && !$stripdangerous) &&
         FindHTMLExploits($message, $id, $ent,
                          $allowiframes, $allowobjects, $allowforms,
                          $allowscripts, $allowwebbugs,
@@ -571,8 +577,8 @@ sub SearchHTMLBody {
     }
     if ($webbugfound) {
       ## Log the <img>
-      #MailScanner::Log::InfoLog("HTML Img tag found in message %s from %s",
-      #  $id, $message->{from});
+      MailScanner::Log::InfoLog("HTML Img tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       #  if MailScanner::Config::Value('logwebbugs', $message);
       # Mark the message
       if ($allowwebbugs) {
@@ -592,16 +598,13 @@ sub SearchHTMLBody {
         #print STDERR "Going to disarm web bugs\n";
       } else {
         #print STDERR "Web Bug ignored\n";
-        #
-        # Don't mark potential web bugs as infections, they are just images
-        # until we analyse them in disinfection.
-        #
-        #$message->{otherreports}{"$attach"} .=
-        #  MailScanner::Config::LanguageValue($message, 'foundwebbug') . "\n";
-        #$message->{othertypes}{"$attach"}   .= "c";
-        #$message->{otherinfected}++;
-        #$message->{silent} = 1 if $silentviruses =~ / HTML-WebBug /i;
-        #$counter++;
+        # Web bugs neither allowed nor converted. So must be stopped.
+        $message->{otherreports}{"$attach"} .=
+          MailScanner::Config::LanguageValue($message, 'foundwebbug') . "\n";
+        $message->{othertypes}{"$attach"}   .= "c";
+        $message->{otherinfected}++;
+        $message->{silent} = 1 if $silentviruses =~ / HTML-WebBug /i;
+        $counter++;
         1;
       }
     }
