@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepOther.pm,v 1.38.2.16 2005/04/27 07:30:58 jkf Exp $
+#   $Id: SweepOther.pm,v 1.38.2.20 2005/12/17 19:04:30 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.38.2.16 $, 10;
+$VERSION = substr q$Revision: 1.38.2.20 $, 10;
 
 # Attributes are
 #
@@ -63,6 +63,7 @@ sub ScanBatch {
   my $ScanType = shift;
 
   # Insert your own checking here.
+  $0 = 'MailScanner: scanning for filenames and filetypes';
 
   # In $BaseDir, you will find a directory for each message, which has the
   # same name as the message id. Also there is a messageid.header file
@@ -164,11 +165,34 @@ sub ScanBatch {
       my $tnefname = $message->{entity2file}{$message->{tnefentity}};
       #print STDERR "TNEF Filename is $tnefname\n";
 
+      my $LogNames = MailScanner::Config::Value('logpermittedfilenames',
+                                                $message);
+
+      # Set up patterns for simple filename real rules files
+      my($allowpatterns, $denypatterns, $allowexists, $denyexists,
+         @allowpatterns, @denypatterns, $megaallow,   $megadeny);
+      $allowpatterns = MailScanner::Config::Value('allowfilenames', $message);
+      $denypatterns  = MailScanner::Config::Value('denyfilenames', $message);
+      $allowpatterns =~ s/^\s+//; # Trim leading space
+      $denypatterns  =~ s/^\s+//;
+      $allowpatterns =~ s/\s+$//; # Trim trailing space
+      $denypatterns  =~ s/\s+$//;
+      @allowpatterns = split(" ", $allowpatterns);
+      @denypatterns  = split(" ", $denypatterns);
+      $allowexists   = @allowpatterns; # Don't use them if they are empty!
+      $denyexists    = @denypatterns;
+      $megaallow     = '(' . join(')|(',@allowpatterns) . ')';
+      $megadeny      = '(' . join(')|(',@denypatterns) . ')';
+      #print STDERR "allowpatterns = $allowpatterns\n";
+      #print STDERR "deny          = $denypatterns\n";
+      #print STDERR "megaallow     = $megaallow\n";
+      #print STDERR "deny          = $megadeny\n";
+
       # Insert new filename rules checking code here
       #print STDERR "Searching for dodgy filenames in $id\n";
       #print STDERR "SafeFile2File = " . %{$message->{safefile2file}} . "\n";
       #while (($attach, $safename) = each %{$message->{file2safefile}}) {
-      while ($safename = $messagefh->read()) {
+      while (defined($safename = $messagefh->read())) {
         #print STDERR "Examinin $id/$safename\n";
         next unless -f "$id/$safename"; # Skip . and ..
         #print STDERR "Real filename of $safename is \"" . $message->{safefile2file}{$safename} . "\"\n";
@@ -176,27 +200,62 @@ sub ScanBatch {
         #print STDERR "Safe filename is $safename\n";
         next if $attach eq "" && $safename eq "";
         #print STDERR "Searching long name \"$attach\" short name \"$safename\"\n";
+        # New for V4. The ?= on the end makes the regexp match
+        # even when the filename is in a foreign character set.
+        # This replaces '$' at the end of the string with "(\?=)?$"
+        $attach =~ s/\$$/(\\\?=)\?\$/;
+
+        #
+        # Implement simple all-matches rulesets for allowing and denying files
+        #
+
+        my $MatchFound = 0;
+        my($logtext, $usertext);
+
+        # Ignore if there aren't any patterns
+        if ($allowexists) {
+          if ($attach =~ /$megaallow/i || $safename =~ /$megaallow/i) {
+            $MatchFound = 1;
+            MailScanner::Log::InfoLog("Filename Checks: Allowing %s %s",
+                                      $id, $safename)
+              if $LogNames;
+          }
+        }
+        # Ignore if there aren't any patterns
+        if ($denyexists) {
+          if (!$MatchFound && ($attach =~ /$megadeny/i ||
+                               $safename =~ /$megadeny/i)) {
+            $MatchFound = 1;
+            # It's a rejection rule, so log the error.
+            $logtext = MailScanner::Config::LanguageValue($message,
+                                                        'foundblockedfilename');
+            $usertext = $logtext;
+            MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
+                                      $logtext, $id, $attach);
+            $message->{namereports}{$safename} .= "$usertext ($safename)\n";
+            $message->{nametypes}{$safename}   .= "f";
+            $counter++;
+            $message->{nameinfected}++;
+          }
+        }
+
+
+
         # Work through the attachment filename rules,
         # using the first rule that matches.
         my($i);
-        my $MatchFound = 0;
         my $FilenameRules = MailScanner::Config::FilenameRulesValue($message);
-        my $LogNames = MailScanner::Config::Value('logpermittedfilenames',
-                                                  $message);
+        next unless $FilenameRules;
 
         #foreach $i (@$FilenameRules) {
         #  print STDERR "FilenameRule: $i\n";
         #}
 
-        for ($i=0; $i<scalar(@$FilenameRules) && !$MatchFound; $i++) {
-          my($allowdeny, $regexp, $logtext, $usertext)
+        my($allowdeny, $regexp);
+        for ($i=0; !$MatchFound && $i<scalar(@$FilenameRules); $i++) {
+          ($allowdeny, $regexp, $logtext, $usertext)
             = split(/\0/, $FilenameRules->[$i]);
 
-          # New for V4. The ?= on the end makes the regexp match
-          # even when the filename is in a foreign character set.
-          # This replaces '$' at the end of the string with "(\?=)?$"
-          $attach =~ s/\$$/(\\\?=)\?\$/;
-
           #print STDERR "Filename match $i: \"$allowdeny\" \"$regexp\" \"$attach\" \"$safename\" $logtext $usertext\n";
           # Skip this rule if the regexp doesn't match
           # Check both filenames, the safe and the nasty. This is for
@@ -231,9 +290,13 @@ sub ScanBatch {
 
   # Don't do these checks if they haven't specified a filetype rules file
   # or they haven't specified a "file" command
-  return $counter if MailScanner::Config::IsSimpleValue('filetyperules') &&
-                     !MailScanner::Config::Value('filetyperules');
   return $counter if !MailScanner::Config::Value('filecommand');
+  return $counter if MailScanner::Config::IsSimpleValue('filetyperules') &&
+                     !MailScanner::Config::Value('filetyperules') &&
+                     MailScanner::Config::IsSimpleValue('allowfiletypes') &&
+                     !MailScanner::Config::Value('allowfiletypes') &&
+                     MailScanner::Config::IsSimpleValue('denyfiletypes') &&
+                     !MailScanner::Config::Value('denyfiletypes');
 
   $counter += CheckFileContentTypes($batch);
 
@@ -347,19 +410,76 @@ sub CheckFileTypesRules {
 
     $tnefname = $message->{entity2file}{$message->{tnefentity}};
 
-    my($i, $MatchFound, $FiletypeRules, $LogTypes);
+    my $LogTypes = MailScanner::Config::Value('logpermittedfiletypes',
+                                              $message);
+
+    # Set up patterns for simple filename real rules files
+    my($allowpatterns, $denypatterns, $allowexists, $denyexists,
+       @allowpatterns, @denypatterns, $megaallow,   $megadeny);
+    $allowpatterns = MailScanner::Config::Value('allowfiletypes', $message);
+    $denypatterns  = MailScanner::Config::Value('denyfiletypes', $message);
+    $allowpatterns =~ s/^\s+//; # Trim leading space
+    $denypatterns  =~ s/^\s+//;
+    $allowpatterns =~ s/\s+$//; # Trim trailing space
+    $denypatterns  =~ s/\s+$//;
+    @allowpatterns = split(" ", $allowpatterns);
+    @denypatterns  = split(" ", $denypatterns);
+    $allowexists   = @allowpatterns; # Don't use them if they are empty!
+    $denyexists    = @denypatterns;
+    $megaallow     = '(' . join(')|(',@allowpatterns) . ')';
+    $megadeny      = '(' . join(')|(',@denypatterns) . ')';
+    #print STDERR "allowpatterns = $allowpatterns\n";
+    #print STDERR "deny          = $denypatterns\n";
+    #print STDERR "megaallow     = $megaallow\n";
+    #print STDERR "deny          = $megadeny\n";
+
+
+    my($i, $FiletypeRules);
     $FiletypeRules = MailScanner::Config::FiletypeRulesValue($message);
-    $LogTypes = MailScanner::Config::Value('logpermittedfiletypes', $message);
 
     while(($safename, $type) = each %$attachtypes) {
       $attach = $message->{safefile2file}{$safename} || $tnefname;
       next if $attach eq "" && $safename eq "";
 
+      #
+      # Implement simple all-matches rulesets for allowing and denying files
+      #
+
+      my $MatchFound = 0;
+      my($logtext, $usertext);
+
+      # Ignore if there aren't any patterns
+      if ($allowexists) {
+        if ($type =~ /$megaallow/i) {
+          $MatchFound = 1;
+          MailScanner::Log::InfoLog("Filetype Checks: Allowing %s %s",
+                                    $id, $safename)
+            if $LogTypes;
+        }
+      }
+      # Ignore if there aren't any patterns
+      if ($denyexists) {
+        if (!$MatchFound && $type =~ /$megadeny/i) {
+          $MatchFound = 1;
+          # It's a rejection rule, so log the error.
+          $logtext = MailScanner::Config::LanguageValue($message,
+                                                        'foundblockedfiletype');
+          $usertext = $logtext;
+          MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
+                                    $logtext, $id, $attach);
+          $message->{namereports}{$safename} .= "$usertext ($safename)\n";
+          $message->{nametypes}{$safename}   .= "f";
+          $counter++;
+          $message->{nameinfected}++;
+        }
+      }
+
       # Work through the attachment filetype rules,
       # using the first rule that matches.
-      $MatchFound = 0;
-      for ($i=0; $i<@$FiletypeRules && !$MatchFound; $i++) {
-        my($allowdeny, $regexp, $logtext, $usertext)
+      next unless $FiletypeRules;
+      my($allowdeny, $regexp);
+      for ($i=0; !$MatchFound && $i<@$FiletypeRules; $i++) {
+        ($allowdeny, $regexp, $logtext, $usertext)
           = split(/\0/, $FiletypeRules->[$i]);
 
         next unless $type =~/$regexp/i;
