@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: TNEF.pm,v 1.12.2.6 2005/09/02 08:57:08 jkf Exp $
+#   $Id: TNEF.pm,v 1.12.2.12 2006/03/02 22:45:24 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -35,12 +35,13 @@ use strict 'vars';
 use strict 'refs';
 no  strict 'subs'; # Allow bare words for parameter %'s
 
+use DirHandle;
 use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.12.2.6 $, 10;
+$VERSION = substr q$Revision: 1.12.2.12 $, 10;
 
 my($UseTNEFModule) = 0;
 
@@ -123,7 +124,7 @@ sub InternalDecoder {
   my($fh, $filename, %parms);
 
   # Make the temporary tnef files be created under /tmp for easy removal.
-  mkdir "/tmp/tnef.$$";
+  mkdir "/tmp/tnef.$$", 0777;
   chmod 0700, "/tmp/tnef.$$";
   %parms = ( ignore_checksum => "true",
              output_dir      => "/tmp/tnef.$$",
@@ -131,7 +132,14 @@ sub InternalDecoder {
   my $tnef = Convert::TNEF->read_in("$dir/$tnefname", \%parms);
   if ($tnef) {
     #print STDERR "Parsing returned something\n";
-    for ($tnef->attachments) {
+    #print STDERR "Attachment list is \"" . $tnef->attachments . "\"\n";
+    #print STDERR "List is \"" . join('","', @{$tnef->attachments}) . "\"\n";
+    $message->{entity}->make_multipart;
+    my $addcontents = 0;
+    $addcontents = 1
+      if MailScanner::Config::Value('replacetnef',$message) =~ /[12]/;
+    my($safename, @replacements);
+    foreach $_ (@{$tnef->attachments}) {
       #print STDERR "Doing attachment $_\n";
       if (my $handle = $_->datahandle) {
         #print STDERR "Have a datahandle $handle\n";
@@ -151,11 +159,35 @@ sub InternalDecoder {
           print $fh $handle->as_string;
         }
         close($fh);
+
+        if ($addcontents) {
+          # Add the member file to the list of attachments in the message
+          #print STDERR "Reading attachment from \"" . $_->name .
+          #             "\" and \"" . $_->longname . "\"\n";
+          #print STDERR "Attachment \"$filename\" --> \"" .
+          #             $_->longname . "\"\n";
+          # Save the new names for logging
+          $safename = $message->MakeNameSafe($_->longname, $dir);
+          push @replacements, $safename;
+          #print STDERR "Safe name is \"$safename\"\n";
+          $message->{entity}->attach(Type => "application/octet-stream",
+                                     Encoding => "base64",
+                                     Disposition => "attachment",
+                                     Filename => $safename,
+                                     Path => $filename);
+          $message->{bodymodified} = 1;
+          #print STDERR "Message = $message and Message ID = " . $message->{id} . "\n";
+        }
       }
     }
     $tnef->purge;
     undef $tnef;
+    $message->{foundtnefattachments} = 1;
+    #$message->{entity}->dump_skeleton();
     system("rm -rf /tmp/tnef.$$");
+    MailScanner::Log::InfoLog("Message %s added TNEF contents %s",
+                              $message->{id}, join(',', @replacements))
+      if @replacements;
     return 1;
   } else {
     # It failed
@@ -240,6 +272,45 @@ sub ExternalDecoder {
     return 1; # Command terminated
   } else {
     return 0 if $TimedOut || $PipeReturn; # Command failed to exit w'success
+
+    # It all worked, so now add everything back into the message.
+    #print STDERR "Dir is \"$dir\" and tnefname is \"$tnefname\"\n";
+
+    return 1
+      unless MailScanner::Config::Value('replacetnef',$message) =~ /[12]/;
+
+    my $dirh = new DirHandle $dir;
+    return 0 unless defined $dirh;
+    my($type, $encoding);
+    $message->{entity}->make_multipart;
+    my($safename, @replacements);
+    while (defined($_ = $dirh->read)) {
+      #print STDERR "Directory entry is \"$_\" in \"$dir\"\n";
+      next unless -f "$dir/$_";
+      next if $_ eq $tnefname;
+      #next if /^msg[\d-]+\.txt$/;
+      if (/^msg[\d-]+\.txt$/) {
+        ($type, $encoding) = ("text/plain", "8bit");
+      } else {
+        ($type, $encoding) = ("application/octet-stream", "base64");
+      }
+      $safename = $message->MakeNameSafe($_, $dir);
+      push @replacements, $safename;
+      $message->{entity}->attach(Type => $type,
+                                 Encoding => $encoding,
+                                 Disposition => "attachment",
+                                 Filename => $safename,
+                                 Path => "$dir/$_");
+      $message->{bodymodified} = 1;
+      $message->{foundtnefattachments} = 1;
+    }
+    undef $dirh;
+    #$message->{entity}->dump_skeleton();
+
+    MailScanner::Log::InfoLog("Message %s added TNEF contents %s",
+                              $message->{id}, join(',', @replacements))
+      if @replacements;
+
     return 1; # Command succeded and terminated
   }
 }
