@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm,v 1.1.2.38 2005/09/19 18:30:20 jkf Exp $
+#   $Id: ZMailer.pm,v 1.1.2.39 2005/11/20 14:53:42 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.38 $, 10;
+$VERSION = substr q$Revision: 1.1.2.39 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -334,6 +334,10 @@ sub ReadQf {
     }
   }
 
+  # Decode the ISO encoded Subject line
+  # Over-ride the default default character set handler so it does it
+  # much better than the MIME-tools default handling.
+  MIME::WordDecoder->default->handler('*' => \&MailScanner::Message::WordDecoderKeep7Bit);
   # Decode the ISO encoded Subject line
   $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
 
